@@ -27,11 +27,11 @@ import gevent
 from gevent import Timeout, Greenlet
 from gevent.socket import create_connection, getfqdn
 from gevent.socket import error as socket_error
-from gevent.queue import Queue, Empty
 
 from slimta.smtp.reply import Reply
 from slimta.smtp.client import Client
 from slimta import logging
+from ..pool import RelayPoolClient
 from . import SmtpRelayError
 
 __all__ = ['SmtpRelayClient']
@@ -40,18 +40,16 @@ log = logging.getSocketLogger(__name__)
 hostname = getfqdn()
 
 
-class SmtpRelayClient(Greenlet):
+class SmtpRelayClient(RelayPoolClient):
 
     def __init__(self, address, queue, socket_creator=None, ehlo_as=None,
                        tls=None, tls_immediately=False,
                        tls_required=False, tls_wrapper=None,
-                       connect_timeout=None, command_timeout=None,
+                       connect_timeout=10.0, command_timeout=10.0,
                        data_timeout=None, idle_timeout=None,
                        credentials=None):
-        super(SmtpRelayClient, self).__init__()
+        super(SmtpRelayClient, self).__init__(queue, idle_timeout)
         self.address = address
-        self.queue = queue
-        self.idle = False
         if socket_creator:
             self._socket_creator = socket_creator
         self.socket = None
@@ -63,8 +61,7 @@ class SmtpRelayClient(Greenlet):
         self.tls_wrapper = tls_wrapper
         self.connect_timeout = connect_timeout
         self.command_timeout = command_timeout
-        self.data_timeout = data_timeout
-        self.idle_timeout = idle_timeout
+        self.data_timeout = data_timeout or command_timeout
         self.credentials = credentials
 
     def _socket_creator(self, address):
@@ -101,7 +98,10 @@ class SmtpRelayClient(Greenlet):
 
     def _authenticate(self):
         with Timeout(self.command_timeout):
-            auth = self.client.auth(*self.credentials)
+            if callable(self.credentials):
+                auth = self.client.auth(*self.credentials())
+            else:
+                auth = self.client.auth(*self.credentials)
         if auth.is_error():
             raise SmtpRelayError.factory(auth)
 
@@ -194,18 +194,8 @@ class SmtpRelayClient(Greenlet):
             if self.client:
                 self.client.io.close()
 
-    def _pop_message(self):
-        self.idle = True
-        try:
-            n, result, envelope = self.queue.get(timeout=self.idle_timeout)
-            return result, envelope
-        except Empty:
-            return None, None
-        finally:
-            self.idle = False
-
     def _run(self):
-        result, envelope = self._pop_message()
+        result, envelope = self.poll()
         if not result:
             return
         try:
@@ -213,13 +203,13 @@ class SmtpRelayClient(Greenlet):
             self._handshake()
             while result:
                 if self._check_server_timeout():
-                    self.queue.put((0, result, envelope))
+                    self.queue.appendleft((result, envelope))
                     break
                 if self._send_envelope(result, envelope):
                     result.set(True)
                 if self.idle_timeout is None:
                     break
-                result, envelope = self._pop_message()
+                result, envelope = self.poll()
         except SmtpRelayError as e:
             result.set_exception(e)
         except Exception as e:
