@@ -22,8 +22,12 @@
 """Package defining the class containing a standard SMTP reply made up of a code
 and a message, as well as a few pre-defined standard replies.
 
+.. _ENHANCEDSTATUSCODES: http://tools.ietf.org/html/rfc2034
+
 """
 
+from __future__ import absolute_import
+
 import re
 
 __all__ = ['Reply', 'unknown_command', 'unknown_parameter', 'bad_sequence',
@@ -189,6 +193,10 @@ class Reply(object):
         else:
             self._esc = value
 
+    @property
+    def raw_message(self):
+        return self._message
+
 
 #: Reply sent when an unknown SMTP command is received by a server.
 unknown_command = Reply('500', '5.5.2 Syntax error, command unrecognized')
