@@ -5,7 +5,7 @@ from mox import MoxTestBase, IsA
 from gevent.socket import socket
 
 from slimta.smtp.client import Client
-from slimta.smtp.auth.mechanisms import Plain
+from slimta.smtp.auth.standard import Plain
 
 
 class TestSmtpClient(MoxTestBase):
